package jp.dip.commonsense.android.mametalk;

import java.text.SimpleDateFormat;
import java.util.Date;

import android.app.Dialog;
import android.app.ListActivity;
import android.database.Cursor;
import android.os.Bundle;
import android.os.DeadObjectException;
import android.provider.Im.BaseMessageColumns;
import android.provider.Im.ContactsColumns;
import android.text.TextUtils;
import android.text.util.Linkify;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.GridView;
import android.widget.ListView;
import android.widget.TextView;

import com.google.android.gtalkservice.IChatSession;

public abstract class AbstractChatActivity extends ListActivity 
						implements View.OnClickListener, View.OnKeyListener {
	//DB邩Kv
	private static final String[] smileyIcons = {":)", ":D", ";)", ":(", ":'(",
												":-o", ":-/", "x-(", ">.<", ":-|",
												"B-)",":P", "<3", "</3", "}:)"};
	private EditText messageText;
	private IChatSession chatSession;
	protected String contactName;
	
	@Override
    public void onCreate(Bundle icicle) {
        super.onCreate(icicle);
        setContentView(R.layout.chat);
        messageText = (EditText) findViewById(R.id.message_edit);
        messageText.setOnKeyListener(this);
        contactName = icicle != null ? icicle.getString(ContactsColumns.USERNAME) : null;
        if (contactName == null) {
        	Bundle extras = getIntent().getExtras();
        	contactName = extras != null ? extras.getString(ContactsColumns.USERNAME) : null;
        }
        setTitle("Talking with " + contactName);
        Button sendButton = (Button) findViewById(R.id.smiley_button);
        sendButton.setOnClickListener(this);
        reflectExtras(icicle);
	}
	
	protected abstract void reflectExtras(Bundle icicle);
	
	@Override
	protected void onListItemClick(ListView l, View v, int position, long id) {
		setSelection(position);
		Cursor cursor = (Cursor)l.getSelectedItem();
		Dialog dialog = new Dialog(this);
		dialog.setContentView(R.layout.message_detail_dialog);
		dialog.setTitle(R.string.dialog_title_message_detail);
		long date = cursor.getLong(cursor.getColumnIndex(BaseMessageColumns.DATE));
		String dispDate = new SimpleDateFormat("dd MMM yyyy HH:mm:ss").format(new Date(date));
		TextView dateText = (TextView)dialog.findViewById(R.id.message_date_view);
		dateText.setText(dispDate);
		TextView bodyText = (TextView)dialog.findViewById(R.id.message_body_view);
		bodyText.setText(cursor.getString(cursor.getColumnIndex(BaseMessageColumns.BODY)));
		Linkify.addLinks(bodyText, Linkify.ALL);
		dialog.show();
	}

	@SuppressWarnings("deprecation")
	@Override
	protected void onResume() {
		super.onResume();
		chatSession = IChatSession.Stub.asInterface(getIntent().getIBinderExtra("chatSession"));
        fillData();
        if (getListAdapter() != null) setSelection(getListAdapter().getCount() - 1);
		messageText.requestFocus();
	}

	@Override
	public boolean onKey(View view, int keyCode, KeyEvent keyevent) {
		if (view.getId() == R.id.message_edit
				&& keyevent.getAction() == KeyEvent.ACTION_DOWN) {
			if (keyCode == KeyEvent.KEYCODE_DPAD_CENTER) {
				String comment = messageText.getText().toString();
		        if (TextUtils.isEmpty(comment)) {
		            //MessageUtils.showMessage(this, getText(R.string.invalid_comment));
		            return false;
		        }
		        if (chatSession == null) {
		        	MessageUtils.showMessage(this, getText(R.string.xmpp_service_not_connected));
		            return false;
		        }
		        try {
		            chatSession.sendTextMessage(messageText.getText().toString());
		            messageText.setText("");
		        } catch (DeadObjectException ex) {
		            Log.e(this.getClass().getName(), "caught " + ex);
		            MessageUtils.showMessage(this, getText(R.string.found_stale_xmpp_service));
		            chatSession = null;
		        }
			}
		}
		return false;
	}

	@Override
	public void onClick(View view) {
		//Smiley̕\B\[X̂Ōv
		final Dialog dialog = new Dialog(this);
		dialog.setTitle(R.string.dialog_title_smiley);
		dialog.setContentView(R.layout.smiley_dialog);
		GridView gv = (GridView)dialog.findViewById(R.id.smiley_grid);
		gv.setAdapter(new ArrayAdapter<String>(this, R.layout.smiley_row, smileyIcons));
		gv.setOnItemClickListener(new AdapterView.OnItemClickListener() {
			@SuppressWarnings({ "unchecked", "synthetic-access" })
			@Override
			public void onItemClick(AdapterView parent, View v, int position, long id) {
				parent.setSelection(position);
				String smiley = (String)parent.getSelectedItem();
				messageText.setText(messageText.getText().toString() + smiley);
				dialog.dismiss();
			}
		});
		dialog.show();
	}
	
	protected abstract void fillData();
}
